{ **********************************************************************
  *                        Program TEXPLOT.PAS                         *
  *                            Version 1.4                             *
  *                    (c) J. Debord, January 2004                     *
  **********************************************************************
  This program creates a TeX file which plots a Fourier series:

  F1 = 0.75 * (1 + Cos(Phi))
  F2 = 2.5 * [1 + Cos(2 * Phi - Pi)]
  F3 = F1 + F2

  Note: this function is used in chemistry to describe the torsional
        energy of an amide bond.

  The resulting file must be processed with LaTeX. The graphics may be
  converted to PostScript using the dvips utility.

  Example: latex figure
           dvips figure
  ********************************************************************** }

program TexDemo;

uses
  FMath, Matrices, PlotVar, TexPlot;

const
  NPT      = 100;
  PIDIV180 = PI / 180;

var
  F : Text;

{$F+}
function Func1(Phi : Float) : Float;
begin
  Func1 := 0.75 * (1.0 + Cos(Phi * PIDIV180));
end;

function Func2(Phi : Float) : Float;
begin
  Func2 := 2.5 * (1.0 + Cos(2.0 * Phi * PIDIV180 - PI));
end;

function Func3(Phi : Float) : Float;
begin
  Func3 := Func1(Phi) + Func2(Phi);
end;
{$F-}

begin
  Xwin1 := 0;
  Ywin1 := 0;
  Xwin2 := 100;
  Ywin2 := 100;

  XAxis.Min  := -180.0;
  XAxis.Max  :=  180.0;
  XAxis.Step :=   45.0;

  YAxis.Min  := 0.0;
  YAxis.Max  := 6.0;
  YAxis.Step := 1.0;

  XTitle.Text := '$\phi$ (deg)';
  YTitle.Text := 'E (kcal/mol)';

  Grid := BOTH_GRID;

  InitTexGraph(F, 'figure.tex');
  WriteXAxis(F);
  WriteYAxis(F);
  WriteGrid(F);

  WriteFunc(F, Func1, XAxis.Min, XAxis.Max, NPT, 5, 'dotted', False);
  WriteFunc(F, Func2, XAxis.Min, XAxis.Max, NPT, 3, 'dashed', False);
  WriteFunc(F, Func3, XAxis.Min, XAxis.Max, NPT, 5, 'solid',  False);

  CloseTexGraph(F);
end.
